;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xb62fc671a64f9bc964c96dc3c0e5427b . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x061b7afbc6873250c6c349045bfe0e39 . "./code.cg2js/cg-printing.fasl")
(#xeb0d61c593db91004af546095fd19162 . "./code.cg2js/cg-caret.fasl")
(#xe05f61c8e155f4c9107f629cb6e5da43 . "./code.cg2js/cg-item-list.fasl")
(#xb02701a6e10cd2be957176e9ed00e9fd . "./code.cg2js/cg-progress-indicator.fasl")
(#x09b9a92f83f208253c108f59727bff68 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xa23aa8b793f29458f82eace7ae3961d1 . "./code.cg2js/cg-outline.fasl")
(#xdaf008012b6e1eb145e7f797d7d6d748 . "./code.cg2js/cg-palette.fasl")
(#x1fc6c22df7e02d3245533b72233ad729 . "./code.cg2js/cg-selected-object.fasl")
(#x8f094889bad78e7e3d62f4495e6ca1ba . "./code.cg2js/cg-status-bar.fasl")
(#x421b3c8fc79a02f83a51e0617e6ea019 . "./code.cg2js/cg-wrap-string.fasl")
(#x6326928db0b92863e3172b14566bd3f6 . "./code.cg2js/cg-fill-texture.fasl")
(#x61d8c7ccf349870ae6dbba4721909998 . "./code.cg2js/cg-layout.fasl")
(#x5678e195d75e81c9c0b5b74d02c08586 . "./code.cg2js/cg-timer.fasl")
(#xfd844f17a241c0a6a689dff1b3d3592e . "./code.cg2js/cg-dialog-item.fasl")
(#x03b1ee68ad67b0522de69e90839e8ab7 . "./code.cg2js/cg-button.fasl")
(#x2bb71cd87e33bb97b9478ea95f672dba . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x1a07748cbb48d93f60c3ad8b7f0e64cc . "./code.cg2js/cg-tray.fasl")
(#x01bb628d14d9883b329787dd36e94c49 . "./code.cg2js/cg-plot-widget.fasl")
(#x9818fa3d797ef3d9848498bee8840759 . "./code.cg2js/cg-pixmap.fasl")
(#x831a057af232c419ff7b88df40b7ad61 . "./code.cg2js/cg-font-dialog.fasl")
(#xb5ca049c3f151acf27efd7cdb25aa128 . "./code.cg2js/cg-os-widget.fasl")
(#x8d32e80554f8e6d13d1ff9d6217a5816 . "./code.cg2js/cg-text-or-combo.fasl")
(#x2e7361066d01a4d8a92952bec58fa9cd . "./code.cg2js/cg-trackbar.fasl")
(#xef7eff0206037cc5639946b6960812e8 . "./code.cg2js/cg-up-down-control.fasl")
(#xd2a67cef49713a5ecbda5ab9fbbc2ac0 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x0185355bde99a8a449701d260768839b . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x98d9a4a2071f1d610c23619ce9ee5cef . "./code.cg2js/cg-intersect-posbox.fasl")
(#x31d0d71f9831552a47b365546592588d . "./code.cg2js/cg-bitmap-stream.fasl")
(#x1be1c18726a80ae5c6bc64d8cc43f7f2 . "./code.cg2js/cg-chart-widget.fasl")
(#x8ca84c6e7d105dfccc0bf8c24a702db7 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x0536f09f7e99632eeb724d28c42139d6 . "./code.cg2js/cg-group-box.fasl")
(#xc7c4ab2d342ce89185ece0256487789f . "./code.cg2js/cg-static-text.fasl")
(#x9e798e6a9ebbe55374ce74ee12afba2a . "./code.cg2js/cg-class-slot-grid.fasl")
(#xc45d734882b0fa998ad7f8c0770db36c . "./code.cg2js/cg-color-dialog.fasl")
(#x7da376ca1710c62ae0f384a7d5b89d7e . "./code.cg2js/cg-intersect.fasl")
(#xc1ba1e8284fd4328c55e7e9916a256ea . "./code.cg2js/cg-gesture-emulation.fasl")
(#xd53304c9546fb2649a709eafb30d799e . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xd6a22c89ac929d0aceef0fd6e442df2b . "./code.cg2js/cg-html-dialog.fasl")
(#x3f6dc70b806afba0902a103712443006 . "./code.cg2js/cg-lettered-menu.fasl")
(#x35f6a270cbe0b751af641445667b8cf2 . "./code.cg2js/cg-class-grid.fasl")
(#x5c00feb2828725474da2b6f182a6ad62 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x3067d9832fa2ff636d1190d10706b94d . "./code.cg2js/cg-calendar.fasl")
(#x81ca035237220e492de6178429818cd1 . "./code.cg2js/cg-editable-text.fasl")
(#x7a8fd412e0c136065ab55bcd28ebed1b . "./code.cg2js/cg-property.fasl")
(#x60e99becb38495d6e40af0561963d50b . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x6f9dea05543b5f831f642978aa9f4d4f . "./code.cg2js/cg-grid-widget.fasl")
(#x626a7b494ea771a2fffd46cb9ea6a978 . "./code.cg2js/cg-paren-matching.fasl")
(#x20db21a8b411e87cc0bcd423b198b63f . "./code.cg2js/cg-dev.fasl")
(#xf6f1fde950df0c504e3027da5b0e7e05 . "./code.cg2js/cg-drawable.fasl")
(#xd13afc311e4836ec74ce0f7f4d4ba404 . "./code.cg2js/cg-curve.fasl")
(#xeca914e3cd0727cbec588e37fc56b1fb . "./code.cg2js/cg-lisp-widget.fasl")
(#xa115bbf01ca1f4773a3a549eda01aea9 . "./code.cg2js/cg-dde-utils.fasl")
(#xd365d6999d044606bc59aeb904e1f3db . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x1c2e424fea2a6af133b50d5655fabffa . "./code.cg2js/cg-object-editor-layout.fasl")
(#xcd1ec7db5c8224444bb8b33af9c24018 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xa2c8adbc11243934356a2220329ee1d8 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x1e2dbf7598f679ed4dd2fd7879975b7e . "./code.cg2js/cg-lamp.fasl")
(#xf6544ebdb26cb96849c5ab23139aa884 . "./code.cg2js/cg-common-control.fasl")
(#x340da57172de765145e57a8ab38987da . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x8f233f96d7866abf05450975a12d218b . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x99a1c7e5b4352d51b778645dc12d2a7b . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x8882f6805b46ed32213f036ea7a70d06 . "./code.cg2js/cg-project-window.fasl")
(#xde946e80d2279051ebe7d3915dd82aa9 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xa5baf0b475f6179081ea12c7846d90de . "./code.cg2js/cg-class-support.fasl")
(#xb0e61975700b33ac6baf8ff3e04318cc . "./code.cg2js/cg-multi-picture-button.fasl")
(#xa3472a7012e3520ff835ae2b1716b186 . "./code.cg2js/cg-yes-no-list.fasl")
(#x3d723edf8a32882695e399dcd9b1844c . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x2e8c372ea9cae6bbadfad3475b0e850e . "./code.cg2js/cg-string-dialog.fasl")
(#x1a4f54ac77dec3a974afba658dc6609a . "./code.cg2js/cg-menu-tooltip.fasl")
(#xab513596b6ad84845625f22d6802646c . "./code.cg2js/cg-dropping-outline.fasl")
(#xc041f01de25ca746dce11398b1604010 . "./code.cg2js/cg.fasl")
(#x413a1c23bdc1f8b5d54b19c1439f52b9 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xd5655f8ca4bac0ff300101242ac1c3ce . "./code.cg2js/cg-scaling-stream.fasl")
(#x60e8b50114faa6e1b6785095d4be3fd6 . "./code.cg2js/cg-yes-no-string.fasl")
(#x4f94a8df4ae1662ea573965ea1120cec . "./code.cg2js/cg-icon-pixmap.fasl")
(#x4b174254a614286b2d36bdfa7f90e679 . "./code.cg2js/cg-web-browser.fasl")
(#x7a5e3fa20ac6f1f72b836320d69e0f93 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x67526322196c54e24947c4dc5286f5e6 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x13c92879175e7e4b23bb57e50c19649a . "./code.cg2js/cg-message-dialog.fasl")
(#xfab3ff7e953b350c04688bf4673666ab . "./code.cg2js/cg-comtab.fasl")
(#xca9d8009894fc6a33c177e6773b11487 . "./code.cg2js/cg-base.fasl")
(#xcb9e8f508feacfccfc29b0a96c91d32c . "./code.cg2js/cg-pixmap-widget.fasl")
(#xaef36697f4c9d9235fdd59a27299e5ca . "./code.cg2js/cg-text-edit-pane.fasl")
(#xfa5cf23aa33ad507323962883a67800a . "./code.cg2js/cg-utility-dialog.fasl")
(#x1de3529a7ed66cb91b66eafc2085b69b . "./code.cg2js/cg-object-editor.fasl")
(#x8fcbda4e7c069add8f8c47dfab863eb0 . "./code.cg2js/cg-edit-in-place.fasl")
(#x0cda4d575d8cd8ae8c3edcf4946118c2 . "./code.cg2js/cg-radio-button.fasl")
(#x93a5846b8b622cb0befe812c1deaf261 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x56f4b921980350200c736f27c8fdfaa0 . "./code.cg2js/cg-nodes.fasl")
(#x876b4634a2e31ec51d7afcd19f3b171c . "./code.cg2js/cg-get-position.fasl")
(#x6669e856d05b124d28d3b46ae02cdc3c . "./code.cg2js/cg-directory-dialog.fasl")
(#x1ea7f80a1a3d7684ea0dd22a3d182c40 . "./code.cg2js/cg-lisp-text.fasl")
(#x6b867dc9da0f83123f724b5396493a21 . "./code.cg2js/cg-combo-box.fasl")
(#xd4d8f276813374a8697534a353e9e7be . "./code.cg2js/cg-text-widget.fasl")
(#xd212a0bd8415bf9a29471e9d711f7bc4 . "./code.cg2js/cg-list-view.fasl")
(#xa08bd39e81f3f73f3ce5763071b8cbbe . "./code.cg2js/cg-graphics-context.fasl")
(#xabe44f750680669eaba1572da621f90f . "./code.cg2js/cg-get-pixmap.fasl")
(#xeb801aea1078683e12cbbc6e8e840665 . "./code.cg2js/cg-menu.fasl")
(#xbd18009cf5fe911238871fa26591327f . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x6c894d0b55d99bf4a509f3872bfca364 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x3c0906b09ca7b35e0b79dd12c5119b6e . "./code.cg2js/cg-tooltip.fasl")
(#x168a0b073ebbef9adb2f9fce309f4222 . "./code.cg2js/cg-html-widget.fasl")
(#x4f5091eeeb3595f9dab7faec646c1843 . "./code.cg2js/cg-choice-list.fasl")
(#x37cca81fb9c1b6aecb50f88d8c989901 . "./code.cg2js/cg-icon.fasl")
(#x557fab6da868ccd0b0aead2fe32b9daa . "./code.cg2js/cg-acache.fasl")
(#x373a98f02997824f710981eca09e8dde . "./code.cg2js/cg-template-string.fasl")
(#x35d3a48a60bb942a5dc280aab056073b . "./code.cg2js/cg-bitmap-pane.fasl")
(#xcfe3f6c54ab27b3352b224c9511b19ad . "./code.cg2js/cg-toggling-widget.fasl")
(#x278a7d3a5d0d511018d88354ece89f8a . "./code.cg2js/cg-tab-control.fasl")
(#x237f4c52cfd80bac80f14416b14fa198 . "./code.cg2js/cg-split-bar.fasl")
(#xa33166a7df6f8ca19bf952135498219e . "./code.cg2js/cg-picture-widget.fasl")
(#x7b5451176f2e053eabe5d5797ec83a38 . "./code.cg2js/cg-player.fasl")
(#xd11394d919134d6ed2abd3aab5485806 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x67400511147c7631ab8478aba740645d . "./code.cg2js/cg-choose-printer.fasl")
(#xba3a5d427a24b1f9e23b52408fff9f47 . "./code.cg2js/cg-os-window.fasl")
(#x01b93cceae26d092d37deee3b6216983 . "./code.cg2js/cg-scroll-bar.fasl")
(#x81c60752fe791e19d0bd84934c4577f7 . "./code.cg2js/cg-clipboard.fasl")
(#x97b18009a5b7dc8475e3de687ced7d99 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x551d7020d1c36478bb0a777943bb816a . "./code.cg2js/cg-hotspot.fasl")
(#x27174e61dfe3d6ed8c1d4d8473dcce35 . "./code.cg2js/cg-check-box.fasl")
(#xb4b5cbf0b2d159db0f3579c6c396e5e1 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xe5ec0dd106c9e9d4aab3afe47dcb8658 . "./code.cg2js/cg-file-dialog.fasl")
(#x08409acc31f788dc9bf55c10646df8ca . "./code.cg2js/cg-toolbar.fasl")
(#xfd45aa857e470119c86201f714c3b00f . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xf7526cf8c8ebcf3a56426e15f1c86497 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x3fbb3358c13958824fff88f305b9eb09 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x72fd8e9acfc362bf8bef1397c8e50c1c . "./code.idejs/ide-project.fasl")
(#x95b2a0975c1af9e9023e6744840e170b . "./code.idejs/ide-intro.fasl")
(#x9c3342858f8fcec517e6cb2f9093068c . "./code.idejs/ide-find-in-files.fasl")
(#xd36f8b40b1e16960932da2a156641f29 . "./code.idejs/ide-grapher.fasl")
(#x16f5db190736bc387898e0b596bed257 . "./code.idejs/ide-inspector.fasl")
(#x2d6adb9b50e4fd66327c3ce358c9edec . "./code.idejs/ide-editor.fasl")
(#x645f8c52fa076a39af6363faa1f56c67 . "./code.idejs/ide-window-list.fasl")
(#x2eb9e5597b8b085cf1cabf03fe1f7878 . "./code.idejs/ide-process-browser.fasl")
(#x9f7a3010602e8e7e593691e9e867af18 . "./code.idejs/ide-patch-dialog.fasl")
(#xa65d4d0ea8a4c4b037f7f9b2dcec665b . "./code.idejs/ide-class-grapher.fasl")
(#x27ab8476e9eba7ed832c57ec3a2eb928 . "./code.idejs/ide-base.fasl")
(#x3e8332a042fbb7fca77ac4e6f09d6fd9 . "./code.idejs/ide.fasl")
(#x711cd17df3e4a407a2a522a7b48c4cac . "./code.idejs/ide-trace.fasl")
(#xd9a08071fc3bba09b59ff396c8dd5619 . "./code.idejs/ide-form.fasl")
(#x09741060fa4dec984500867dda84d0ca . "./code.idejs/ide-find-definitions.fasl")
(#x82dbd6a320a3105d54784912ff2108c5 . "./code.idejs/ide-apropos.fasl")
(#xd2b5c6fcc7967160b7ea31645f0a5400 . "./code.idejs/ide-class-browser.fasl")
(#xec3d8b7400d5be0cbff3b85e469815b6 . "./code.idejs/ide-navigator.fasl")
(#xf92d76aa6f6e118bab509978589d2738 . "./code.idejs/ide-clipboard.fasl")
(#x83e363fdc7db9ca18f6f84dc4cf245a4 . "./code.idejs/ide-acache.fasl")
(#xe1db3759685ff08a74a1d94fa11ee81d . "./code.idejs/ide-keyshort.fasl")
(#xbc9884e38582c1a138378f0d2ff5e525 . "./code.idejs/ide-package-list.fasl")
(#x2fdcafa58b5134ecd9b44db3918e761d . "./code.idejs/ide-dev.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
